// ADC.c
// Denne er basert p en blanding av eksempelfiler fra "c programming for microcontrollers" (Joe Pardue, 2005) 
// og butterfly-demoprogramvaren fra atmel.

#include "PC_Comm.h"
#include "ADC.h"
#include "BCD.h"


//__flash 
int LIGHT_ADC[] = // Table used to find the Vref, when using the voltage-reading function 
        { 
            0x35,0x60,0x80,0x0B0,0x11D,0x13D,0x15A,0x17A,0x197,0x1B9,0x1DA,
            0x1F9,0x216,0x240,0x26D,0x282,0x2A2,0x2EF,0x332,0x3B0,0x3F2
        };
		
//__flash 
float LIGHT_VOLTAGE[] = // Vref table correspondent to the LIGHT_ADC[] table
        {
            2.818,2.820,2.824,2.827,2.832,2.835,2.839,2.841,2.843,2.847,2.850,
            2.853,2.857,2.863,2.867,2.870,2.874,2.882,2.893,2.917,2.939
        };
		
float Vref = 2.900; // initial value

//WHERE DOES THIS GO? 
	/*First, we set the ATmega169 ADC multiplexer to connect
	the ADC to pin 61 (ADC0) using the ADMUX register (databook p 207).
	The Butterfly has the ATmega169 pin 62, (AREF) connected to a bypass
	capacitor to help lessen noise on the ADC, so we set the ADMUX 
	bits 6 and 7 to 0 to select the 'use external reference' option and 
	set all the rest to select the ACD multiplexer input channel.*/
	
	
void init_ADC()
{
	// disable JTAG:
	unsigned char sreg;
	sreg = SREG;
	cli(); //disable interrupt
	
	MCUCR |= (1<<JTD); //set jtag disable bit, must be done twice in four cycles according to databook
	MCUCR |= (1<<JTD);
	
	SREG = sreg;
	sei(); //enable interrupt
	//JTAG disabled
	
	ADC_init(1);
	
}


/**********************************************************************************
	Volt Meter 
***********************************************************************************/
float read_ADC(int adcNr)
{
    int ADCresult = 0;
    int ADCresult_temp = 0;
	
    char Temp;
    unsigned char i = 0;
    char TL;
    char TH;    
    char VL;

    float V_ADC;
	float returnvoltage;
    char VoltageHB;
    char VoltageLB;
	
	switch (adcNr)
	{
		case '1':
			ADMUX = 4; //ADC1 er koblet til JTAG1 p butterfly
			break;
		case '2':
			ADMUX = 6; //ADC2 er koblet til JTAG3 p butterfly
			break;
		case '3':
			ADMUX = 5; //ADC3 er koblet til JTAG5 p butterfly
			break;
		case '4':
			ADMUX = 7; //ADC4 er koblet til JTAG9 p butterfly
			break;
	}
    
    ADCresult = ADC_read();         // Find the ADC value

//  Do a Light-measurement first to determine the Vref, 
//  because the LDR affects the Vref 

        ADCresult_temp = ADCresult;     // Store the ADCresult from the voltage reading
          
        ADC_init(LIGHT_SENSOR);         // Init the ADC to measure light
            
        ADCresult = ADC_read();         // Read the light value

        // Find Vref
        for (i=0; i<=22; i++)
        {
            if (ADCresult <= LIGHT_ADC[i])
            {
                break;
            }
        }
        if(!i)              // if it's very bright
            Vref = 2.815;
        else if(i >= 21)
            Vref = 2.942;   // if it's totally dark
        else
            Vref = LIGHT_VOLTAGE[i];   
          
        ADMUX = VOLTAGE_SENSOR;     
        ADCresult = ADCresult_temp; // Get the ADCresult from the voltage reading

// Light-measurement finished 

       V_ADC = ( ADCresult * Vref ) / 1024; // Calculate the voltage
           
        //V_ADC = ( V_ADC * 6 );      // Multiply by 6 cause of the voltage division
		
		returnvoltage = V_ADC;
            
        VoltageHB = V_ADC;              // Store the high-byte
        V_ADC = ( V_ADC - VoltageHB );
        VoltageLB = ( V_ADC * 100 );    // Store the low-byte
     
        Temp = CHAR2BCD2(VoltageHB);    // Convert from char to bin
    
        TL = (Temp & 0x0F) + '0';
        TH = (Temp >> 4) + '0';
		
		Temp = CHAR2BCD2(VoltageLB);    // Convert from char to bin
   
        VL = (Temp >> 4) + '0';
	
	// Send the voltage to the PC
	sendString("The reading is ");
	sendChar(TL);
	sendChar('.');
	sendChar(VL);
	sendString(" volts.\r");	
	
	return returnvoltage;
}	

/**********************************************************************************
	ADC common functions
***********************************************************************************/

void ADC_init(char input)
{
	/*First, we set the ATmega169 ADC multiplexer to connect
	the ADC to pin 61 (ADC0) using the ADMUX register (databook p 207).
	The Butterfly has the ATmega169 pin 62, (AREF) connected to a bypass
	capacitor to help lessen noise on the ADC, so we set the ADMUX 
	bits 6 and 7 to 0 to select the 'use external reference' option and 
	set all the rest to select the ACD multiplexer input channel.*/
    ADMUX = input;    // external AREF and ADCx

    // set ADC prescaler to , 1MHz / 8 = 125kHz    
    ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);    

    //  Take a dummy reading , which basically allows the ADC 
    // to hack up any hairballs before we take any real readings
    input = ADC_read();       
}

int ADC_read(void)
{
    char i;
    int ADC_temp;
    
    // To save power, the voltage over the LDR and the NTC is turned off when not used
    // This is done by controlling the voltage from a I/O-pin (PORTF3)
    sbi(PORTF, PORTF3);     // Enable the VCP (VC-peripheral)
    sbi(DDRF, PORTF3);          

    sbi(ADCSRA, ADEN);     // Enable the ADC

    //do a dummy readout first
    ADCSRA |= (1<<ADSC);        // do single conversion
    while(!(ADCSRA & 0x10));    // wait for conversion done, ADIF flag active
        
    for(i=0;i<8;i++)            // do the ADC conversion 8 times for better accuracy 
    {
        ADCSRA |= (1<<ADSC);        // do single conversion
        while(!(ADCSRA & 0x10));    // wait for conversion done, ADIF flag active
        
        ADC_temp = ADCL;            // read out ADCL register
        ADC_temp += (ADCH << 8);    // read out ADCH register        

        ADC += ADC_temp;      // accumulate result (8 samples) for later averaging
    }

    ADC = ADC >> 3;     // average the 8 samples
        
    cbi(PORTF, PORTF3);     // disable the VCP
    cbi(DDRF, PORTF3);  
    
    cbi(ADCSRA, ADEN);      // disable the ADC

    return ADC;
}



