//PWM.c
#include "PC_Comm.h"
#include "PWM.h"


int speed = 1;
int maxPW [4] = {1700,1200,1900,2100};
int minPW [4] = {900,1040,900,900};
int currentCtc [4];

void init_PWM() //initieringen
{

	currentCtc[0] = 1500;
	currentCtc[1] = 1500;
	currentCtc[2] = 1500;
	currentCtc[3] = 1500;

	
	//initialiser OC1A for servo 1/3
	sbi(DDRB, 5); //sett OC1A som output
	sbi(PORTB, 5); //set OC1A high
	
	//initier OC1B for servo 2/4
	sbi(DDRB, 6); //sett OC1B som output 
	sbi(PORTB, 6); //set OC1B high
  	 
}

//behandling av ctc (pulsbredde) sendt fra pc
//basert p "c programming for microcontrollers" (Joe Pardue, 2005)
int parse_ctc(char s[], int servoNr) 
{
	char ctc[11];
	unsigned char i = 6, j = 0;
	int ctc_int = 0;

	while( (s[i] != '\0') && (j <= 11) ) 
	{
	
		if( (s[i] >= '0') && (s[i] <= '9') )
		{
			ctc[j++] = s[i++];
		}
		else
		{
			sendString("Error - Parse_ctc received a non integer: ");
			sendChar(s[i]);
			sendChar('\r');
			return 0;
		}
	}
	
	ctc[j] = '\0';
	
	
	ctc_int = atoi(ctc);
	if(ctc_int<minPW[servoNr-1] || ctc_int>maxPW[servoNr-1])//pwm-pulsbredde m vre mellom min og max for servoen
	{
		sendString("Error - Parse_ctc must be within servo bounds");
		return 0;
	}
	else
	{
		set_ctc(ctc_int, servoNr);
	}
		
	return 1;
}

//behandling av frekvens sendt fra pc, felles for servoene
//basert p "c programming for microcontrollers" (Joe Pardue, 2005)
int parse_freq(char s[]) 
{
	char freq[11];
	unsigned char i = 4, j = 0;

	while( (s[i] != '\0') && (j <= 11) ) 
	{
	
		if( (s[i] >= '0') && (s[i] <= '9') )
		{
			freq[j++] = s[i++];
		}
		else
		{
			sendString("Error - Parse_freq received a non integer: ");
			sendChar(s[i]);
			sendChar('\r');
			return 0;
		}
	}
	
	freq[j] = '\0';
	
	set_freq(atoi(freq));
		
	return 1;
}

void set_position(double pos, int servoNr){ //setter posisjon i prosent


	int diff = maxPW[servoNr-1] - minPW[servoNr-1];
	double pos_ctc = minPW[servoNr-1] + diff*(pos/100);
	set_ctc(pos_ctc, servoNr);

}

void increment_position(int servoNr){
	if (currentCtc[servoNr-1] < maxPW[servoNr-1] ){ //m jo ikke kjre for langt
	
		currentCtc[servoNr-1] = currentCtc[servoNr-1] + speed;
		set_ctc(currentCtc[servoNr-1], servoNr);
	}
}

void decrement_position(int servoNr){
	if (currentCtc[servoNr-1] > minPW[servoNr-1]){ //m jo ikke kjre for langt
	
		currentCtc[servoNr-1] = currentCtc[servoNr-1] - speed;
		set_ctc(currentCtc[servoNr-1], servoNr);
	}
}

void set_ctc(int PWMcount, int servoNr) //starter PWM-signal med pulsbredde p "PWMcount" microsekunder
{
	
	currentCtc [servoNr-1] = PWMcount;
	
	// WGM10-13 setter pwm-modus phase and frequency correct, ICR1 top.
	// COM1A1=1: clear OC1A when upcounting, set when downcounting; tilsvarende med COM1B1.
	TCCR1A = (1<<COM1A1)|(1<<COM1B1);//|(0<<WGM11)|(0<<WGM10); 
	TCCR1B = (1<<WGM13);//|(0<<WGM12);
	TCCR1C = (0<<FOC1A)|(0<<FOC1B); //FOC1A m vre 0 for  kunne bruke pwm
	
	sbi(TCCR1B, CS10);         // start Timer1, prescaler(1)
	
	
	
		//deler opp PWMcount for  kunne skrive til 16-bits register
		int temp_hi = PWMcount;
		temp_hi >>= 8; //right shift 8 for  f hy del
		int temp_lo = PWMcount;
		
	if (servoNr==1 || servoNr ==3){
		//sendString("servonr 1 eller 3.\r");
	
		OCRA1H=temp_hi;
		OCRA1L=temp_lo;
		
		
	} else if (servoNr ==2 || servoNr == 4){
		//sendString("servonr 2 eller 4.\r");
		
		OCRB1H=temp_hi;
		OCRB1L=temp_lo;
		
	}
	
}

void set_freq(int freqcount) //setter frekvens i Hz
{
	char freq[11];

	sendString("Setting the frequency to: ");
	itoa(freqcount,freq,10);	
	sendString(freq);
	sendChar('\r');
	
	//bestem riktig timervalue for PWM-frekvens:
	int PWMfrequency = freqcount;//300;
	int timervalue = 1000000 / PWMfrequency / 2;
	
	//del opp timervalue for  f satt rett verdi til ICR1:
	int temp_hi = timervalue;
	temp_hi >>= 7; //right shift 8 for  f hy del, left shift 1 for  kompensere for 2mHz klokke i stedet for 1mHz
	int temp_lo = timervalue;
	temp_lo <<=1; //kompenser for 2MHz i stedet for 1MHz klokke
	
	//clear Timer1 count;
	TCNT1H = 0;
	TCNT1L = 0;
	
	//sett timervalue til ICR1:
	ICR1H= temp_hi;
	ICR1L= temp_lo;

}

void stop_PWM(void)
{
	cbi(TCCR1B,0); //stop playing
	TCCR1A = 0;
	TCCR1B = 0;
	sbi(PORTB, 5); //set OC1A high
}


SIGNAL(SIG_OUTPUT_COMPARE0) //avruddsbehandling hvorfor? lar den st. fra "c programming for microcontrollers" (Joe Pardue, 2005)
{
	if(PORTD &= 1) cbi(PORTD, 0);
	else sbi(PORTD, 0);

}
